import json

import pytest
from hippocrates.questionnaires.models import create_questions, create_results


@pytest.fixture
def questionnaire_set_json():
    """ Load in fixtures
    """

    with open('tests/fixtures/question_set.json') as json_file:
        question_set_json = json.load(json_file)
    return question_set_json


@pytest.fixture
def questionnaire_set(questionnaire_set_json):
    """ Load in fixtures
    """
    return create_results(question_set_json=questionnaire_set_json), \
        create_questions(questionnaire_set_json)



"""
Use this to create a parent class all hippocrates can use
"""

import typing as t
from pathlib import Path

from hippocrates.questionnaires.base import Assessment
from hippocrates.questionnaires.models import (create_questions,
                                               create_results,
                                               import_question_set)

JSON_QUESTION_SET = version_path = Path(__file__).parent / \
                               'question_set.json'

question_set_json = import_question_set(path=JSON_QUESTION_SET)
question_set = create_questions(question_set_json)


class PHQ2Assessment(Assessment):

    def __init__(self):
        raw_json = import_question_set(path=JSON_QUESTION_SET)
        self.question_set: t.List = create_questions(raw_json)
        self.total_questions = len(self.question_set)
        self.results: t.List = create_results(question_set_json=raw_json)



from hippocrates.questionnaires.bdi.assessment import BDIAssessment
from hippocrates.questionnaires.gad7.assessment import GAD7Assessment
from hippocrates.questionnaires.phq9.assessment import PHQ9Assessment
from hippocrates.questionnaires.rosenberg_self_esteem.assessment import \
    RosenbergSelfEsteemAssessment
from pick2 import pick

OPTIONS = {
    'Beck Depression Index': BDIAssessment(),
    'GAD 7': GAD7Assessment(),
    'PHQ 9': PHQ9Assessment(),
    'Rosenberg Self Esteem': RosenbergSelfEsteemAssessment(),
}
PICK_OPTIONS = list(OPTIONS.keys())


def pick_assessment():
    option, index = pick(
        PICK_OPTIONS, 'Please select an assessment from the list:',
        indicator='=>', default_index=0,
    )
    return OPTIONS[option]


if __name__ == '__main__':
    assessment = pick_assessment()
    result = assessment.take_assessment()
    print(result.severity)



from hippocrates.questionnaires.gad2.assessment import GAD2Assessment


def test_bdi_assessment():
    gad2_assessment = GAD2Assessment()
    assert gad2_assessment.question_set


def test_bdi_questions():
    gad2_assessment = GAD2Assessment()
    assert gad2_assessment.questions()



"""
Use this to create a parent class all hippocrates can use
"""

import typing as t

from hippocrates.questionnaires.models import Answer, QuestionAnswerSet, Result
from pick2 import pick


class Assessment:

    question_set: t.List = []
    results: t.List = []

    def questions(self):
        return self.question_set

    def ask_question(self, options: t.List[Answer], question_from_set:
                     QuestionAnswerSet):
        option, index = pick(
            options, question_from_set.question.text,
            indicator='=>', default_index=0,
        )
        question_from_set.answer = question_from_set.answer_options[index]

    def take_assessment(self) -> Result:
        for question_from_set in self.question_set:
            options: t.List = []
            for answer in question_from_set.answer_options:
                options.append(answer.text)

            self.ask_question(options, question_from_set)

        return self.result()

    def result(self) -> Result:
        total = 0
        for question in self.question_set:
            if not question.answer:
                # raise exception?
                continue
            total = total + question.answer.value

        for result in self.results:
            if result.min <= total <= result.max:
                return result
        raise Exception

    def complete(self):
        """
        check if all questions have an answer
        :return:
        """
        count = 0
        for question in self.question_set:
            if question.answer:
                count = count + 1

        return True if count == self.total_questions else False



from unittest.mock import MagicMock

import pytest
from hippocrates.questionnaires.base import Assessment
from hippocrates.questionnaires.models import (Answer, Question,
                                               QuestionAnswerSet, Result)


def test_assessment_complete():
    assessment = Assessment()

    question = MagicMock()
    question.answer = True
    assessment.total_questions = 1
    assessment.question_set = [question]
    assert assessment.complete()


def test_assessment_result():
    assessment = Assessment()

    question_one = Question('Do you like chips?')
    question_two = Question('Do you like dip?')

    answer_one = Answer('yes', 10)
    answer_two = Answer('no', 3)

    question_set_one = QuestionAnswerSet(question=question_one,
                                         answer_options=[answer_one],
                                         answer=answer_one)
    question_set_two = QuestionAnswerSet(question=question_two,
                                         answer_options=[answer_two],
                                         answer=answer_two)

    assessment.total_questions = 1
    assessment.question_set = [question_set_one, question_set_two]

    result_one = Result(min_score=0, max_score=5, severity='good',
                        comment='OK')
    result_two = Result(min_score=5, max_score=20, severity='not good',
                        comment='NOT OK')
    assessment.results = [result_one, result_two]
    assert assessment.result() == result_two

    # question_set_one = QuestionAnswerSet(question=question_one,
    #                                      answer_options=[answer_one])
    # question_set_two = QuestionAnswerSet(question=question_two,
    #                                      answer_options=[answer_two])


def test_take_assessment(questionnaire_set):
    assessment = Assessment()
    assessment.total_questions = 2
    assessment.results = questionnaire_set[0]
    assessment.question_set = questionnaire_set[1]

    def ask_question(options, question):
        question.answer = question.answer_options[0]

    assessment.ask_question = ask_question
    assessment.take_assessment()
    assert assessment.complete()


def test_result_failure(questionnaire_set):
    assessment = Assessment()
    assessment.total_questions = 5
    # assessment.results = questionnaire_set[0]
    assessment.question_set = questionnaire_set[1]

    # def ask_question(options, question):
    #     question.answer = question.answer_options[0]
    #
    # assessment.ask_question = ask_question
    # assessment.take_assessment()
    with pytest.raises(Exception):
        assessment.result()



import json
import typing as t
from pathlib import Path
from uuid import uuid4


class Question:

    def __init__(self, text: str,) -> None:
        self.text = text
        self.key = uuid4()


class Answer:

    def __init__(self, text: str, value: int,) -> None:
        self.text = text
        self.value = value


class Result:

    def __init__(
        self, min_score: int, max_score: int, severity: str,
        comment: str,
    ) -> None:
        self.min = min_score
        self.max = max_score
        self.severity = severity
        self.comment = comment


class ResultSet:

    def __init__(self, results: t.List[Result]) -> None:
        self.results = results

    def __inter__(self):
        return self.results


class QuestionAnswerSet:

    def __init__(
        self, question: Question, answer_options: t.List[Answer],
        answer=None,
    ) -> None:
        self.question = question
        self.answer_options = answer_options
        self.answer = answer


class Questionnaire:

    def __init__(self, question_answer_set: t.List[QuestionAnswerSet]) -> None:
        self.question_answer_set = question_answer_set

    def questions(self):
        return iter(self.question_answer_set)


def import_question_set(path: Path) -> t.Dict:
    with open(path) as json_file:
        question_set_json = json.load(json_file)
    return question_set_json


def create_questions(question_set_json):
    """
    Convert Questions and Answers from the raw json
    :param question_set_json:
    :return:
    """
    questionnaire = []
    for order_key, question_answer in \
            question_set_json['question_set'].items():
        question = Question(question_answer['question'])
        answers = _build_answers(question_answer['answers'])
        question_answer_set = QuestionAnswerSet(
            question=question,
            answer_options=answers,
            answer=None,
        )
        questionnaire.append(question_answer_set)
    return questionnaire


def create_results(question_set_json):
    """
    Convert Questions and Answers from the raw json
    :param question_set_json:
    :return:
    """
    results = []
    for result_json in question_set_json['results']:
        results.append(
            Result(
                    min_score=result_json['min'],
                    max_score=result_json['max'],
                    comment=result_json['comment'],
                    severity=result_json['severity'],
            ),
        )
    return results


def _build_answers(answers):
    answers_set = []
    for answer, value in answers.items():
        answers_set.append(Answer(text=answer, value=value))

    return answers_set



from hippocrates.questionnaires.models import Questionnaire, create_questions


def test_create_questions(questionnaire_set_json):
    question_set = create_questions(questionnaire_set_json)
    assert len(question_set) == len(questionnaire_set_json['question_set'])


def test_questionnaire_iteration(questionnaire_set_json):
    question_set = create_questions(questionnaire_set_json)
    questionnaire = Questionnaire(question_set)

    for question in questionnaire.questions():
        assert hasattr(question, 'answer_options')
        assert hasattr(question, 'question')



from hippocrates.questionnaires.rosenberg_self_esteem.assessment import \
    RosenbergSelfEsteemAssessment


def test_bdi_assessment():
    rosenberg_self_esteem_assessment = RosenbergSelfEsteemAssessment()
    assert rosenberg_self_esteem_assessment.question_set



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hippocrates",
    version="0.1.0",
    author="Chris Hannam",
    author_email="ch@chrishannam.co.uk",
    description="Mental health questionnaires.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/chrishannam/hippocrates",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pick>=0.6.4',
    ],
)
